/*
rndinit.c  Version 1.0.0
Initialize random number generator seed to date
and time in seconds.
Resulting seed is unpredictable.
Copyright (C) 2003-2004 dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

    Free Software Foundation, Inc.
    59 Temple Place - Suite 330
    Boston, MA  02111-1307, USA.
*/

/* The following copyright notice is for the call */
/* to getmd5. */

/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
rights reserved.

License to copy and use this software is granted provided that it
is identified as the "RSA Data Security, Inc. MD5 Message-Digest
Algorithm" in all material mentioning or referencing this software
or this function.

License is also granted to make and use derivative works provided
that such works are identified as "derived from the RSA Data
Security, Inc. MD5 Message-Digest Algorithm" in all material
mentioning or referencing the derived work.

RSA Data Security, Inc. makes no representations concerning either
the merchantability of this software or the suitability of this
software for any particular purpose. It is provided "as is"
without express or implied warranty of any kind.

These notices must be retained in any copies of any part of this
documentation and/or software.
 */

/*
Usage:
        unsigned char *rndSeed;
        rndSeed = (unsigned char *) rndinit();
*/

/*
System subroutine calls:
    time
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "md5glbl.h"
#include "md5.h"

#define SEED_SZ 16

unsigned char *rndinit(void)
   {
   int i;
   time_t lTime,*ptrTime;
   unsigned char tmpSeed[32];
   unsigned char *rndSeed;
   unsigned char *p,*q;
   void getmd5(unsigned char *seed,
      int seedlen, unsigned char *digest);

   rndSeed = (unsigned char *) malloc (SEED_SZ+16);
   if (rndSeed == NULL)
      {
      fprintf(stderr,"rndinit: Out of memory "
         "allocating rndSeed\n");
      exit(1);
      } /* out of mem */

   p = tmpSeed;
   q = tmpSeed + SEED_SZ;
   while (p < q) *p++ = '\0';

   time (&lTime);

   p = tmpSeed;
   for (i=0;i<4;i++)
      {
      ptrTime  = &lTime+3;
      *p++ = (unsigned char) *ptrTime--;
      *p++ = (unsigned char) *ptrTime--;
      *p++ = (unsigned char) *ptrTime--;
      *p++ = (unsigned char) *ptrTime;
      } /* for each 4 bytes of the seed */

   /* Make seed more random by hashing it. */
   /* RSA Data Security, Inc. */
   /* MD5 Message-Digest Algorithm */
   /* Input is a 16 byte date-time stamp */
   /* Output is a 16 byte message digest */
   /* The MD5 algorithm is described in */
   /* RFC 1321 (April 1992). */
   /* The getmd5 subroutine is derived from */
   /* the RSA Data Security, Inc. */
   /* MD5 Message-Digest Algorithm. */
   getmd5(tmpSeed,SEED_SZ,rndSeed);

   /* The seed must be an odd number. */
   *rndSeed |= 1;
   return rndSeed;
   } /* rndinit */
